--------------------------------------------------------
-- Namespaces
--------------------------------------------------------
local MCL, core = ...;
core.Config = {};

local MCL_Load = core.Config;
local MCL;

--------------------------------------
-- Defaults (usually a database!)
--------------------------------------
local defaults = {
	theme = {
		r = 0, 
		g = 0.8, -- 204/255
		b = 1,
		hex = "00ccff"
	}
}


function MCL_Load:Toggle()
	local menu = MCL or MCL_Load:CreateMenu();
	MCL:SetShown(not MCL:IsShown());
end
function MCL_Load:GetThemeColor()
	local c = defaults.theme;
	return c.r, c.g, c.b, c.hex;
end
function MCL_Load:CreateIcons(point, xOffset, yOffset, id, expID, collected)
	local mount = CreateFrame("Button", nil, expID, "BackdropTemplate")
	mount:SetPoint(point, expID, xOffset, yOffset)
	mount:SetSize(38, 38)
	mount.text = mount:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	mount.text:SetFont("Fonts\\FRIZQT__.TTF", 10)
	mount.text:SetPoint("BOTTOM", mount, "CENTER", 0, 0)
	mount.tex = mount:CreateTexture()
	mount.tex:SetAllPoints(mount)
	mount.tex:SetTexture(GetItemIcon(id))
	if collected then
		mount:SetBackdrop({
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 39,
		})
		mount:SetBackdropBorderColor(0, 1, 0)
	else
		mount.tex:SetVertexColor(0.75, 0.75, 0.75, 0.3);	
	end
	--Adding a tooltip for mounts
	local item, itemLink = GetItemInfo(id);
	mount:HookScript("OnEnter", function()
		if (itemLink) then
			GameTooltip:SetOwner(mount, "ANCHOR_TOP")
			GameTooltip:SetHyperlink(itemLink)
			GameTooltip:Show()
			mount:SetHyperlinksEnabled(true)

		end
	end)
	mount:HookScript("OnLeave", function()
		GameTooltip:Hide()
	end)		
	return mount;
end
function MCL_Load:CreateMountIcons(point, xOffset, yOffset, expID, collected, icon, pos)
	local mount = CreateFrame("Button", nil, expID, "BackdropTemplate")
	mount:SetPoint(point, expID, xOffset, yOffset)
	mount:SetSize(38, 38)
	mount.text = mount:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	mount.text:SetFont("Fonts\\FRIZQT__.TTF", 10)
	mount.text:SetPoint("BOTTOM", mount, "CENTER", 0, 0)
	mount.tex = mount:CreateTexture()
	mount.tex:SetAllPoints(mount)
	mount.tex:SetTexture(icon)
	if collected then
		mount:SetBackdrop({
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 39,
		})
		mount:SetBackdropBorderColor(0, 1, 0)
	else
		mount.tex:SetVertexColor(0.75, 0.75, 0.75, 0.3);	
	end
	--Adding a tooltip for mounts
	return mount;
end
function MCL_Load:CategoryFrame(expT, relativeFrame, xOffset, yOffset)
	--Creating a frame to place expansion content in.
	local categorie = CreateFrame("Frame", nil, relativeFrame);
	categorie:SetSize(800, 90);
	categorie:SetPoint("TOPLEFT", relativeFrame,  xOffset, yOffset);

	--Creating a frame to place title of expansion
	categorie.title = categorie:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	categorie.title:SetPoint("TOPLEFT", 10, 0)
	categorie.title:SetText(expT)
	return categorie;
end
local function ScrollFrame_OnMouseWheel(self, delta)
	local newValue = self:GetVerticalScroll() - (delta * 50);
	
	if (newValue < 0) then
		newValue = 0;
	elseif (newValue > self:GetVerticalScrollRange()) then
		newValue = self:GetVerticalScrollRange();
	end
	
	self:SetVerticalScroll(newValue);
end

local function Tab_OnClick(self)
	PanelTemplates_SetTab(self:GetParent(), self:GetID());

	local scrollChild = MCL.ScrollFrame:GetScrollChild();
	if(scrollChild) then
		scrollChild:Hide();
	end

	MCL.ScrollFrame:SetScrollChild(self.content);
	self.content:Show();
	MCL.ScrollFrame:SetVerticalScroll(0);
end

local function SetTabs(frame, numTabs, ...)
	frame.numTabs = numTabs;
	local contents = {};
	local frameName = frame:GetName();

	for i = 1, numTabs do

		local tab = CreateFrame("Button", frameName.."Tab"..i, frame, "CharacterFrameTabButtonTemplate");
		tab:SetID(i);
		tab:SetText(select(i, ...));
		tab:SetScript("OnClick", Tab_OnClick);

		tab.content = CreateFrame("Frame", nil, MCL.ScrollFrame);
		tab.content:SetSize(760, 500);
		tab.content:Hide();

		table.insert(contents, tab.content);

		if (i == 1) then
			tab:SetPoint("TOPLEFT", MCL, "BOTTOMLEFT", 5, 7);
		else
			tab:SetPoint("TOPLEFT", _G[frameName.."Tab"..(i-1)], "TOPRIGHT", -14, 0);
		end
	end

	Tab_OnClick(_G[frameName.."Tab1"]);

	return unpack(contents);
end

function MCL_Load:insertMountFromID(mountID, rowC, x, category, mountFrame, a, isFactionSpecific, faction, isCollected, playerFaction)
	if (isFactionSpecific == true and faction == playerFaction) or isFactionSpecific == false then
		if rowC > 0 then
			x = x+48
		end
		rowC = rowC+1							
		mountFrame = self:CreateMountIcons("TOPLEFT", x, y, a, isCollected, icon)
		collectableMounts = collectableMounts+1
	end
	if isCollected == true then
		totalc = totalc+1
		totalCollected = totalCollected+1
		if category ~= "Blizzard Store" then
			totalCollectedNoStore = totalCollectedNoStore+1
		end
	end
	complete = true
	return rowC, x, complete
end		

function MCL_Load:insertMountFromItem(ID, rowC, x, category, mountFrame, a, mountID, isFactionSpecific, faction, isCollected, playerFaction)					
	if (isFactionSpecific == true and faction == playerFaction) or isFactionSpecific == false then
		if (isFactionSpecific == true) then
		end
		if rowC > 0 then
			x = x+48
		end
		rowC = rowC+1
		u = u+1
		mountFrame = self:CreateIcons("TOPLEFT", x, y, ID, a, isCollected)
		collectableMounts = collectableMounts+1												
		if isCollected == true then
			totalc = totalc+1
			totalCollected = totalCollected+1
			if category ~= "Blizzard Store" then
				totalCollectedNoStore = totalCollectedNoStore + 1
			end
		end
		local item, itemLink = GetItemInfo(mountID);
		mountFrame:SetScript("OnClick", function(self, button, down)
			if (itemLink) then
				print(itemLink)
			end
		end)
	--This will determine when to start a new row of mounts. Currently set to 6 mounts per row.								
		if(rowC == 50) then
			y = y-45
			x = 10
			rowC = 0
		end
		total = total+1
	end
	complete = true
	return rowC, x, complete		
end

function MCL_Load:CreateMenu()
	--Master Frame for addon
	MCL = CreateFrame("Frame", "MLCFrame", UIParent, "BasicFrameTemplateWithInset");
	MCL:SetMovable(true)
	MCL:EnableMouse(true)
	MCL:RegisterForDrag("LeftButton")
	MCL:SetScript("OnDragStart", MCL.StartMoving)
	MCL:SetScript("OnDragStop", MCL.StopMovingOrSizing)
	MCL:SetFrameStrata("DIALOG")

	--MCL Frame settings
	MCL:SetSize(1200, 600); -- width, height
	MCL:SetPoint("CENTER", UIParent, "CENTER"); -- point, relativeFrame, relativePoint, xOffset, yOffset
	MCL:EnableMouse(true)
	MCL:SetHyperlinksEnabled(true)
	MCL:SetScript("OnHyperlinkClick", ChatFrame_OnHyperlinkShow)

	--Creating title for frame
	MCL.title = MCL:CreateFontString(nil, "OVERLAY", "GameFontHighlight");
	MCL.title:SetPoint("LEFT", MCL.TitleBg, "LEFT", 5, 0);
	MCL.title:SetText("Mount Collection Log");

	MCL.ScrollFrame = CreateFrame("ScrollFrame", nil, MCL, "UIPanelScrollFrameTemplate");
	MCL.ScrollFrame:SetPoint("TOPLEFT", MCL.Bg, "TOPLEFT", 4, -8);
	MCL.ScrollFrame:SetPoint("BOTTOMRIGHT", MCL.Bg, "BOTTOMRIGHT", -3, 4);
	MCL.ScrollFrame:SetClipsChildren(true);
	MCL.ScrollFrame:SetScript("OnMouseWheel", ScrollFrame_OnMouseWheel);	

	MCL.ScrollFrame.ScrollBar:ClearAllPoints();
	MCL.ScrollFrame.ScrollBar:SetPoint("TOPLEFT", MCL.ScrollFrame, "TOPRIGHT", -12, -18);
	MCL.ScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", MCL.ScrollFrame, "BOTTOMRIGHT", -7, 18);

	local playerFaction
	local playerFactionStr = UnitFactionGroup("player")
	if playerFactionStr == "Alliance" then
		playerFaction = 1
	else
		playerFaction = 0
	end

	totalCollected = 0
	totalCollectedNoStore = 0

	local classic, tbc, wotlk, cata, mop, wod, legion, bfa, sl, racial, professions, pvp, worldevents, promotion, other = SetTabs(MCL, 15, "Classic", "TBC", "WOTLK", "CATA", "MOP", "WOD", "LEGION", "BFA", "SL", playerFactionStr, "Professions", "PVP", "WorldEvents", "Promotion", "Other");

	local sections = {classic, tbc, wotlk, cata, mop, wod, legion, bfa, sl, racial, professions, pvp, worldevents, promotion, other}
	local sectionStrings = {"Classic", "TBC", "WOTLK", "CATA", "MOP", "WOD", "LEGION", "BFA", "SL", playerFactionStr, "Professions", "PVP", "WorldEvents", "Promotion", "Other"}

-- update "frame" with new information to display
	local mountFrames = {}
	collectableMounts = 0
	u = 0
	--Creating a frame to place expansion content in.
	--For each value section and category run this loop
	for k,v in pairs(sectionNames) do
		--Reset location of frames
		local yTwo = -20
		local xTwo = 20
		rowC = 0
		local frameName = MCL:GetName()
		--Cycle through each section and check if database entry matches any string.
		for s = 1, #sectionStrings do
			if v.name == sectionStrings[s] then	
				--If string matches a section then we loop through all categories for that section and create a frame.	
				for i = 1, #v.category do
					local a = self:CategoryFrame(v.category[i], sections[s], xTwo, yTwo)
					rowC = 0
					x = 10
					y = -25
					for b,n in pairs(mountList) do
						if n.name == sectionStrings[s] then
							for h,j in pairs(n) do
								if j.name == v.category[i] then
									if j.mounts then
										total = 0
										totalc = 0
										for t = 1, #j.mountID do
											if j.mountID[t] then											
												local _, _, icon, _, _, _, _, isFactionSpecific, faction, _, isCollected, mountID = C_MountJournal.GetMountInfoByID(j.mountID[t])
												rowC, x, complete = self:insertMountFromID(j.mountID[t], rowC, x, v.category[i], mountFrames[u], a, isFactionSpecific, faction, isCollected, playerFaction)
											end					
										end												
										for t = 1, #j.mounts do
											if C_MountJournal.GetMountFromItem(j.mounts[t]) then
												local mountID = C_MountJournal.GetMountFromItem(j.mounts[t])
												local _, _, icon, _, _, _, _, isFactionSpecific, faction, _, isCollected, mountID = C_MountJournal.GetMountInfoByID(mountID)
												rowC, x, complete = self:insertMountFromItem(j.mounts[t], rowC, x, v.category[i], mountFrames[u], a, mountID, isFactionSpecific, faction, isCollected, playerFaction)	
											end								
										end
										a.title:SetText(v.category[i].." ("..totalc.."/"..total..")")
									end
								end
							end
						end
					end
					yTwo = yTwo - 80
				end
			end
		end
	end			
	MCL.title:SetText("Mount Collection Log | Total Collected "..totalCollected.." | Total Without Store Mounts ".. totalCollectedNoStore.." | Total Collectable Mounts "..collectableMounts);
	MCL:Hide();
	return MCL;
end